#include <Windows.h>
#include <objbase.h>
#include <stdio.h>
#include "abspdf.h"


// Generic function to check the success of an operation.  If the
// operation failed, this function prints the formatted error message
// and returns 0.  Otherwise, it returns 1.
//
int check_success(HRESULT hr) {

    // If we had success, simply return 1.
    //
    if (SUCCEEDED(hr)) {
        return 1;
    }

    char *szError; // Error message buffer.

    // Try to format the message and print it out.
    //
    if(FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
                     FORMAT_MESSAGE_FROM_SYSTEM,
                     NULL,
                     hr,
                     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                     (LPSTR)&szError, 
                     0, 
                     NULL) != 0)
    {

        fprintf(stderr, "Error: %s\n", szError);
        LocalFree(szError);

    } else {

        // If, for some reason, we could not format the error,
        // let the user know.
        //
        fprintf(stderr, "Unknown error!\n");

    }

    return 0; // indicate failure

}

int get_serial(IUnknown* pIUnknown) {
 
    HRESULT hr;
    int retval;

    // Look up the IDispatch Interface.
    //
    IDispatch* pIDispatch = NULL;
    hr = pIUnknown->QueryInterface(IID_IDispatch, (void**) &pIDispatch);
    if (!check_success(hr)) {
        return 0;
    }
    
    // Extra checking.
    //
    if (pIDispatch == NULL) {
        fprintf(stderr, 
            "Unknown error getting the IDispatch interface.\n");
        return 0;
    }

    OLECHAR* szFunction[] = { OLESTR("GetSerial") }; 
    DISPID dispid = DISPID_UNKNOWN;
    hr = pIDispatch->GetIDsOfNames( IID_NULL, 
                                    szFunction, 
                                    1, 
                                    LOCALE_SYSTEM_DEFAULT, 
                                    &dispid);

    // If we successfully looked up the method, go on to invoke it.
    //
    if (check_success(hr)) {

        // Create storage for the return value and initialize it.
        //
        VARIANT   varResult;    
        VariantInit( &varResult );

        EXCEPINFO excepInfo;    // storage for exceptions.
        UINT      uArgErr = 0;  // storage for argument errors.

        // Invoke the method.
        //
        DISPPARAMS noParams = {NULL, NULL, 0, 0};
        hr = pIDispatch->Invoke(dispid, 
                                IID_NULL, 
                                LOCALE_USER_DEFAULT, 
                                DISPATCH_METHOD, 
                                &noParams, 
                                &varResult, 
                                &excepInfo, 
                                &uArgErr);

        // Make sure the call succeeded, and that the result is a
        // BSTR before displaying it.
        // 
        if ( check_success(hr) ) {
       
            // Make sure we got the right return type.
            //
            if (varResult.vt != VT_BSTR) {
                fprintf(stderr, 
                    "Bogus return value (not VT_BSTR) from function.\n");
                retval = 0;
            } else {
                wprintf( 
                    OLESTR("\nThe unique machine ID is %s\n"), varResult.bstrVal);
                retval = 1;
            }

        } else {
            retval = 0;
        }

        // Clear the storage we allocated for the result.
        //
        VariantClear( &varResult );

    } else {
        retval = 0;
    }

    // Release the IDispatch pointer.
    //
    if (pIDispatch != NULL) {
        pIDispatch->Release();
        pIDispatch = NULL;
    }

    return retval;
}

extern "C"
int __cdecl
wmain(int argc, wchar_t *argv[]) {

    HRESULT hr = S_FALSE;

    // Initialize the COM library.
    //
    hr = CoInitialize(NULL);  

    if( check_success(hr) ) {

        // Declare a pointer of type IUnknown.  The CoCreateInstance 
        // function makes this point to an instance of the control
        //
        IUnknown* pIUnknown = NULL;

        // Invoke CoCreateInstance to launch control
        //
        hr = CoCreateInstance( CLSID_absPDF_Creator, 
                               NULL, 
                               CLSCTX_INPROC_SERVER, 
                               IID_IUnknown, 
                               (void**)&pIUnknown);

        if( check_success(hr) ) {

            // Extra checking to make sure we have a pointer here.
            //
            if (pIUnknown == NULL) {
                fprintf(stderr, 
                    "Unknown error getting the IUnknown interface.\n");
                return 0;
            }
                
            if (check_success(hr)) {

                get_serial(pIUnknown);

            }

            if (pIUnknown != NULL) {
        
                // Release the object we created.
                // 
                pIUnknown->Release();
                pIUnknown = NULL;
            }

        }
    }

    CoUninitialize();

    return 0;

}

